@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2020
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -105,7 +106,6 @@ sub draw_label {
 
 package X3d::File;
 
-
 use Polymake::Struct (
    [ new => '$' ],
    [ '@ISA' => 'Element' ],
@@ -116,9 +116,9 @@ use Polymake::Struct (
 
 sub coord_axes {
    my ($self,$x)=@_;
-   my $ahead_l = $arrowheadlength * $scale; 
-   my $ahead_r = $arrowheadwidth * $scale;
-   my $cyl_r = $lines_thickness * $scale * 0.8;
+   my $ahead_l = $arrowhead_length * $scale; 
+   my $ahead_r = $arrowhead_radius * $scale;
+   my $cyl_r = $edge_radius * $scale * 0.8;
    my $cyl_h = 4;
    my $ahead_tr = new Vector<Float>([0, ($cyl_h+$ahead_l)/2, 0]);
    my @colors = ('1 0 0','0 0 1','0 1 0');   # blenders coord axes colors
@@ -225,7 +225,6 @@ sub toString {
    my $title = $self->title // "unnamed";
    
    my $x = XML::Writer->new( OUTPUT => 'self', DATA_MODE => 1, DATA_INDENT => 3 );
-
    $x->xmlDecl('UTF-8');
    $x->doctype('X3D', "ISO//Web3D//DTD X3D 3.0//EN", "http://www.web3d.org/specifications/x3d-3.0.dtd");
    $x->startTag('X3D', profile => "Interchange", version => "3.0", 'xmlns:xsd' => "http://www.w3.org/2001/XMLSchema-instance", 'xsd:noNamespaceSchemaLocation' => "http://www.web3d.org/specifications/x3d-3.0.xsd");
@@ -270,12 +269,12 @@ sub init {
    my $self = shift;
    my $P = $self->source;
    $self->coords = embed_3d($P->Vertices); #TODO reuse?: that sub is probably somewhere
-   my $thickness = $P->VertexThickness // 1;
+   my $thickness = $P->VertexThickness;
    my $labels = $P->VertexLabels;
-   my $style = $P->VertexStyle // 1;
+   my $style = $P->VertexStyle;
    foreach (0..scalar(@{$P->Vertices})-1) {
       my $thick = is_code($thickness) ? $thickness->($_) : $thickness;
-      my $radius = ($style =~ $Visual::hidden_re) ? 0 : $scale * $thick * $points_thickness;
+      my $radius = ($style =~ $Visual::hidden_re) ? 0 : $scale * $point_radius * $thick;
       push @{$self->radii}, $radius;
       my $label = defined($labels) ? $labels->($_) : undef;
       push @{$self->label_widths}, $scale * ($avg_char_width * $font_size * length($label) + $text_spacing ) + $radius;
@@ -288,6 +287,19 @@ sub new {
    $self;
 }
 
+sub draw_def {
+   #unused atm
+   my ($self,$writedefto,$x)=@_;
+   my $P=$self->source;
+   
+   $x->startTag('Shape');
+		$x->startTag($writedefto, solid => "false");
+			$x->emptyTag('Coordinate', DEF => "pointCoords", 
+			                           point => join(' ', map { Visual::print_coords($_) } @{$self->coords}));
+		$x->endTag($writedefto);
+	$x->endTag('Shape');
+}
+
 
 sub draw_point {
    my ($self,$x,$coord,$radius,$color,$label)=@_;
@@ -296,8 +308,7 @@ sub draw_point {
    #the vertex sphere
    if ($radius) {
       $x->startTag('Shape');
-         $x->startTag('Sphere', radius => $scale*$points_thickness);
-         $x->endTag('Sphere');
+         $x->emptyTag('Sphere', radius => $radius);
          $x->startTag('Appearance');
             $x->emptyTag('Material', diffuseColor => $color);
          $x->endTag('Appearance');
@@ -311,53 +322,78 @@ sub draw_point {
 }
 
 sub draw_edge {
-   my ($self,$x,$edge,$cyl_r,$cyl_col,$label,$arrow_dir,$ahead_col)=@_; 
+   my ($self,$x,$edge,$cyl_r,$edge_color,$label,$arrow_dir,$ahead_col)=@_; 
    my ($f, $t) = $arrow_dir!=-1 ? @$edge : reverse @$edge;
-   my $fr = @{$self->coords}[$f];
-   my $to = @{$self->coords}[$t];
-   my $d = norm($to - $fr);
-   my $ahead_l = (!$arrow_dir) ? 0 : $arrowheadlength * $scale; 
-   my $ahead_r = $arrowheadwidth * $scale;
-   my $to_r = @{$self->radii}[$t];
-   my $fr_r = @{$self->radii}[$f];
-   my $fr_rr = sqrt(max(0,$fr_r**2-$cyl_r**2));
-   my $cyl_h = $d - $ahead_l - $fr_rr - $to_r;
-   my $maxy = ($cyl_h/2 + $to_r + $ahead_l);
-   my $q = $maxy/$d;
-   my $tr = $fr + (1-$q)*($to-$fr);
-
-   my $rot_fr = new Vector<Float>([0, $maxy, 0]);
-   my $rot_to = $q*($to-$fr);
-   my $ahead_tr = new Vector<Float>([0, ($cyl_h+$ahead_l)/2, 0]);
-   my $raxis = ($rot_fr + $rot_to)/2;
-   my $label_offset = new Vector<Float>([0,0,0]);
-   $x->startTag('Transform',  translation => Visual::print_coords($tr)); 
-      $x->startTag('Transform',  rotation=> Visual::print_coords($raxis)." ".$PI);
-	   if ($ahead_l) {
-         $x->startTag('Transform',  translation => Visual::print_coords($ahead_tr)); 
-   	      $x->startTag('Shape');
-   		      $x->startTag('Appearance');
-   			      $x->emptyTag('Material', diffuseColor => $ahead_col);
-   			   $x->endTag('Appearance');
-   			   $x->emptyTag('Cone', bottomRadius => $ahead_r,
-   			                  		height => $ahead_l);
-   		   $x->endTag('Shape');
-   	   $x->endTag('Transform');
-      }
-            $x->startTag('Shape');
-   	      $x->startTag('Appearance');
-   			   $x->emptyTag('Material', diffuseColor => $cyl_col);
-   			$x->endTag('Appearance');
-   			$x->emptyTag('Cylinder',   radius => $cyl_r,
-   			                  		   height => $cyl_h);
-   		$x->endTag('Shape');
-   	$x->endTag('Transform');
+	
+	my $ahead_l = (!$arrow_dir) ? 0 : $arrowhead_length * $scale; 
+	my $ahead_r = $arrowhead_radius * $scale;
+	my $fr = @{$self->coords}[$f];
+	my $to = @{$self->coords}[$t];
    
-   if (defined($label) && $label!~/^\s*$/) {
-      $self->draw_label($x,$label,$label_offset,'0 0 0');
-   }
+   if ($ahead_l || !$use_lines || defined($label)) {
+		my $to_r = @{$self->radii}[$t];
+		my $fr_r = @{$self->radii}[$f];
+		my $to_offset = $ahead_l ? $to_r : sqrt(max(0,$to_r**2-$cyl_r**2));
+		my $fr_offset = sqrt(max(0,$fr_r**2-$cyl_r**2));
+	
+		my $diff = $to - $fr;
+		my $dist =  norm($diff);
+		my $length = $dist - $fr_offset - $to_offset;
+		my $cyl_h = $length - $ahead_l;
+		my $min_y = $cyl_h/2;
+		
+		my $q = ($min_y + $fr_offset)/$dist;	
+		my $translation = $fr + $q * $diff;
+	
+		my $rot_from = new Vector<Float>([0, - $min_y - $fr_offset, 0]);
+		my $rot_to = $q * (-$diff);
+		my $rot_axis = ($rot_from + $rot_to)/2;
+		my $label_offset = new Vector<Float>([0,0,0]);
+	
+		$x->startTag('Transform',  translation => Visual::print_coords($translation)); 
+			if ($ahead_l || !$use_lines) {
+				$x->startTag('Transform',  rotation=> Visual::print_coords($rot_axis)." ".$PI);
+				if ($ahead_l) {
+					my $ahead_translation = new Vector<Float>([0, ($cyl_h+$ahead_l)/2, 0]);
+					$x->startTag('Transform',  translation => Visual::print_coords($ahead_translation)); 
+						$x->startTag('Shape');
+							$x->startTag('Appearance');
+								$x->emptyTag('Material', diffuseColor => $ahead_col);
+							$x->endTag('Appearance');
+							$x->emptyTag('Cone', bottomRadius => $ahead_r,
+														height => $ahead_l);
+						$x->endTag('Shape');
+					$x->endTag('Transform');
+				}
+
+				if (!$use_lines) {
+						$x->startTag('Shape');
+						$x->startTag('Appearance');
+							$x->emptyTag('Material', diffuseColor => $edge_color);
+						$x->endTag('Appearance');
+						$x->emptyTag('Cylinder',   radius => $cyl_r,
+															height => $cyl_h);
+					$x->endTag('Shape');
+				}
+			}
+			$x->endTag('Transform');
+
+		if (defined($label) && $label!~/^\s*$/) {
+			$self->draw_label($x,$label,$label_offset,'0 0 0');
+		}
+		$x->endTag('Transform');
+	}	
+	if ($use_lines) {
+		$x->startTag('Shape');
+			$x->startTag('Appearance');
+				$x->emptyTag('Material', diffuseColor => $edge_color);
+			$x->endTag('Appearance');
+			$x->startTag('LineSet', vertexCount => "2");
+				$x->emptyTag('Coordinate', point=> Visual::print_coords($fr)." ".Visual::print_coords($to));
+			$x->endTag('LineSet');
+		$x->endTag('Shape');
+	}
 
-	$x->endTag('Transform');
 }
 
 sub draw_points {
@@ -394,41 +430,9 @@ package X3d::Solid;
 
 use Polymake::Struct [ '@ISA' => 'PointSet' ];
 
-sub draw_def {
-   my ($self,$x)=@_;
-   my $P=$self->source;
-   
-   # NOTES ON THE BLENDER IMPORT:
-   # does not resolve prototypes properly
-   # strangely reverts the X axis...
-   
-   # quote /usr/share/blender/scripts/addons/io_scene_x3d/import_x3d.py    line 1878  ff:
-   #     # we need unflattened coord array here, while
-   #     # importmesh_readvertices uses flattened. can't cache both :(
-   #     # todo: resolve that somehow, so that vertex set can be effectively
-   #     # reused between different mesh types?
-   # end of quote
-   # thus, we can't use a coordinate def in, for example, a pointset if we want to 
-   # use it in an indexedfaceset
-   
-   $x->startTag('Shape');
-		$x->startTag('IndexedFaceSet', solid => "false");
-			$x->emptyTag('Coordinate', DEF => "pointCoords", 
-			                           point => join(' ', map { Visual::print_coords($_) } @{$self->coords}));
-		$x->endTag('IndexedFaceSet');
-	$x->endTag('Shape');
-}
 
 sub draw_facet {
    my ($self,$x,$facet,$color,$transparency,$label)=@_;
-   $x->startTag('Shape');
-      $x->startTag('Appearance');
-		   $x->emptyTag('Material', diffuseColor => $color, transparency => $transparency);
-		$x->endTag('Appearance');
-		$x->startTag('IndexedFaceSet', solid => "false", coordIndex => $facet);
-			$x->emptyTag('Coordinate', USE => "pointCoords");
-		$x->endTag('IndexedFaceSet');
-	$x->endTag('Shape');
    
    my $label_offset = new Vector<Float>([0,0,0]);
    if (defined($label) && $label!~/^\s*$/) {
@@ -447,19 +451,33 @@ sub draw_facets {
    my $style = $P->FacetStyle;
    my $labels = $P->FacetLabels;
    my $transparency = $P->FacetTransparency // 0;
-   my $color = $P->FacetColor;
-   my $static_color; 
-   if (defined($color) && !is_code($color)) {
-      $static_color=$color;
+   my $color = $P->FacetColor // $Visual::Color::facets;
+   if (is_code($transparency)) {
+      print "X3d: code FacetTransparency is not supported at the moment\n";
+      $transparency = $transparency->(0);
    }
 
-   foreach my $i (0..scalar(@facets)-1) {
-      my $col = $static_color // $color->($i) // $Visual::Color::facets;
-      my $tp = (is_code($transparency)) ? $transparency->($i) : $transparency;
-      my $label = defined($labels) ? $labels->($i) : undef;
-      $col = $col->toFloat;
-      $self->draw_facet($x,$facets[$i],$col,$tp,$label);  
-   }
+   $x->startTag('Shape');
+      $x->startTag('Appearance');
+         if (!is_code($color)) {
+		      $x->emptyTag('Material', diffuseColor => $color->toFloat, transparency => $transparency);
+         } else {
+		      $x->emptyTag('Material', transparency => $transparency);
+         }
+		$x->endTag('Appearance');
+		$x->startTag('IndexedFaceSet', solid => "true", coordIndex => join(" -1 ", @facets), colorPerVertex => "false");
+			$x->emptyTag('Coordinate', point => join(' ', map { Visual::print_coords($_) } @{$self->coords}));
+         if (is_code($color)) {
+            my @colors;
+            foreach my $i (0..scalar(@facets)-1) {
+               my $col = $color->($i) // $Visual::Color::facets;
+               push @colors, $col->toFloat;
+            }
+			   $x->emptyTag('Color', color => join(' ', @colors));
+         }
+		$x->endTag('IndexedFaceSet');
+	$x->endTag('Shape');
+   
 }
 
 sub draw_edges {
@@ -473,7 +491,7 @@ sub draw_edges {
    my $color = $P->EdgeColor // Visual::get_RGB($Visual::Color::edges);
    $color = $color->toFloat;
 
-   my $radius = $thickness * $lines_thickness * $scale;
+   my $radius = $scale * $edge_radius * $thickness; 
    
    # there is probably a better way to get an edge iterator?
    my $G = graph::graph_from_cycles($P->Facets);
@@ -485,7 +503,6 @@ sub draw_edges {
 sub draw {
    my ($self,$x)=@_;
    $self->draw_points($x) unless $self->source->VertexStyle =~ $Visual::hidden_re;
-   $self->draw_def($x);
    $self->draw_edges($x) unless $self->source->EdgeStyle =~ $Visual::hidden_re;
    $self->draw_facets($x) unless $self->source->FacetStyle =~ $Visual::hidden_re;
    $x;
@@ -520,7 +537,7 @@ sub draw_edges {
 
    for (my $e=$G->all_edges; $e; ++$e) {
       my $thick = (is_code($thickness)) ? $thickness->($e) : $thickness;
-      my $radius = $thick * $lines_thickness * $scale;
+      my $radius = $scale * $edge_radius * $thick;
       my $label = defined($labels) ? $labels->($e) : undef;
       next unless $radius || $label;
       my $col = $static_color // $color->($e) // Visual::get_RGB($Visual::Color::edges);
@@ -533,6 +550,9 @@ sub draw_edges {
 
 sub draw {
    my ($self,$x)=@_;
+   if ($use_lines) { 
+   	$self->draw_def('IndexedLineSet',$x); 
+   }
    $self->draw_points($x) unless $self->source->VertexStyle =~ $Visual::hidden_re;
    $self->draw_edges($x) unless $self->source->EdgeStyle =~ $Visual::hidden_re;
    $x;
