@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2020
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -24,42 +25,41 @@
 
 namespace polymake { namespace polytope {
 
-
-void matroid_polytope(perl::Object m,  perl::OptionSet options )
+void matroid_polytope(BigObject m,  OptionSet options)
 {
-  const Array< Set<int> > bases=m.give("BASES");
-  const int n_bases=bases.size();
-  const int n_elements=m.give("N_ELEMENTS");
+  const Array<Set<Int>> bases = m.give("BASES");
+  const Int n_bases = bases.size();
+  const Int n_elements = m.give("N_ELEMENTS");
   
-  Matrix<Rational> V(n_bases,n_elements+1);
+  Matrix<Rational> V(n_bases, n_elements+1);
   
   //test for each subset of size r
-  for (int b=0; b<n_bases; ++b) {
-    V(b,0)=1;
-    for (auto i=entire(bases[b]); !i.at_end(); ++i)
-      V(b,(*i)+1)=1;
+  for (Int b = 0; b < n_bases; ++b) {
+    V(b,0) = 1;
+    for (auto i = entire(bases[b]); !i.at_end(); ++i)
+      V(b, (*i)+1) = 1;
   }
 
   bool ineq_flag = options["inequalities"];
   if (ineq_flag && m.give("CONNECTED") && n_elements>1) {
-     perl::Object lattice_obj = m.give("LATTICE_OF_FLATS");
+     BigObject lattice_obj = m.give("LATTICE_OF_FLATS");
      graph::Lattice<graph::lattice::BasicDecoration, graph::lattice::Sequential> lattice(lattice_obj);
-     const int size( lattice.nodes()-2 ); //do not use the bottom and top node
-     const int rank=m.give("RANK");
+     const Int size = lattice.nodes()-2; // do not use the bottom and top node
+     const Int rank = m.give("RANK");
      Matrix<Rational> I(size+2*n_elements,n_elements+1);
      Matrix<Rational> E(1,n_elements+1);
-     int f(0);
-     for (int j=1 ; j<rank; ++j) {
+     Int f = 0;
+     for (Int j = 1; j < rank; ++j) {
         for (auto fi=entire(lattice.nodes_of_rank(j)); !fi.at_end(); ++fi, ++f) {
-           I(f,0)=j;
-           for (auto i=entire(lattice.face(*fi)); !i.at_end(); ++i)
-              I(f,(*i)+1)=-1;
+           I(f,0) = j;
+           for (auto i = entire(lattice.face(*fi)); !i.at_end(); ++i)
+              I(f, (*i)+1) = -1;
         }
      }
      // hypersimplex
      // 0 <= x_i <= 1 + sum x_i = rank :
-     E(0,0)=-rank;
-     for (int i=0;i<n_elements;++i) {
+     E(0,0) = -rank;
+     for (Int i = 0; i < n_elements; ++i) {
         I(size+2*i,i+1)=1;
         I(size+2*i+1,0)=1;
         I(size+2*i+1,i+1)=-1;
