@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2020
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -21,22 +21,22 @@
 namespace polymake { namespace polytope {
 
 template <typename Scalar>
-perl::Object scale(perl::Object p_in, const Scalar& factor,
-                   bool store_reverse_transformation)
+BigObject scale(BigObject p_in, const Scalar& factor,
+                bool store_reverse_transformation)
 {
-   int amb_dim=p_in.call_method("AMBIENT_DIM");
+   Int amb_dim = p_in.call_method("AMBIENT_DIM");
    Matrix<Scalar> T(diag(1,(factor*unit_matrix<Scalar>(amb_dim))));
-   return transform<Scalar>(p_in,T,store_reverse_transformation);
+   return transform<Scalar>(p_in, T, store_reverse_transformation);
 }
 
 
 template <typename Scalar>
-perl::Object translate(perl::Object p_in, const Vector<Scalar>& trans,
-                       bool store_reverse_transformation)
+BigObject translate(BigObject p_in, const Vector<Scalar>& trans,
+                    bool store_reverse_transformation)
 {
-   int amb_dim=p_in.call_method("AMBIENT_DIM");
-   Matrix<Scalar> T((unit_vector<Scalar>(amb_dim+1,0)|(trans/unit_matrix<Scalar>(amb_dim))));
-   return transform<Scalar>(p_in,T,store_reverse_transformation);
+   Int amb_dim = p_in.call_method("AMBIENT_DIM");
+   Matrix<Scalar> T((unit_vector<Scalar>(amb_dim+1, 0) | (trans/unit_matrix<Scalar>(amb_dim))));
+   return transform<Scalar>(p_in, T, store_reverse_transformation);
 }
 
 UserFunctionTemplate4perl("# @category Transformations"
