@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2015
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2020
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -16,11 +17,10 @@
 
 #define USE_DDF_
 #include "polymake/polytope/cdd_interface_impl.h"
-#include <sstream>
 
 namespace polymake { namespace polytope { namespace cdd_interface {
 
-template <> inline
+template <>
 double cdd_lp_sol<double>::optimal_value() const
 {
    return double(ptr->optvalue[0]);
@@ -28,7 +28,8 @@ double cdd_lp_sol<double>::optimal_value() const
 
 template class cdd_matrix<double>;
 template class cdd_polyhedron<double>;
-template class solver<double>;
+template class ConvexHullSolver<double>;
+template class LP_Solver<double>;
 
 } } }
 
