@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2020
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -20,9 +21,8 @@ use feature 'state';
 
 package Polymake::Core::PropertyType;
 
-declare @delimiters;
-declare $nesting_level;
 declare $trusted_value;
+declare $nested_instantiation=0;
 
 sub canonical_fallback { }
 sub equal_fallback { $_[0] == $_[1] }
@@ -30,6 +30,8 @@ sub isa_fallback : method { UNIVERSAL::isa($_[1], $_[0]->pkg) }
 sub coherent_type_fallback { undef }
 sub parse_fallback : method { croak( "no string parsing routine defined for class ", $_[0]->full_name ) }
 sub toString_fallback { "$_[0]" }
+sub serialize_fallback { $_[0] }
+sub deserialize_meth : method { $_[0]->construct->($_[1]) }
 
 sub init_fallback { }
 sub performs_deduction { 0 }
@@ -41,11 +43,12 @@ sub generic { undef }
 
 use Polymake::Struct (
    [ new => '$$$;$' ],
-   [ '$name | full_name | mangled_name | xml_name' => '#1' ],
+   [ '$name | full_name | mangled_name' => '#1' ],
    [ '$pkg' => '#2' ],
    [ '$application' => '#3' ],
-   [ '$extension' => '$Application::extension' ],
+   [ '$extension' => '$Extension::loading' ],
    [ '$super' => '#4' ],
+   [ '$params' => 'undef' ],
    [ '$dimension' => '0' ],
    [ '&canonical' => '->super || \&canonical_fallback' ],          # object => void
    [ '&equal'  => '->super || \&equal_fallback' ],                 # object1, object2 => bool
@@ -53,9 +56,9 @@ use Polymake::Struct (
    [ '$upgrades' => 'undef' ],                                     # optional hash PropertyType => 1/0/-1
    [ '&coherent_type' => '->super || \&coherent_type_fallback' ],  # object of a different type => PropertyType of this or a derived type or undef
    [ '&toString' => '->super || \&toString_fallback' ],            # object => "string"
-   [ '&toXML' => '->super' ],                                      # object, XMLwriter, opt. attributes =>
-   [ '&toXMLschema' => 'undef || \&Polymake::Core::XMLwriter::type_toXMLschema' ],          # XMLwriter, opt. attributes =>
-   [ '&XMLdatatype' => '->super' ],                                # => "string" referring to a XML Schema datatype or a pattern
+   [ '&serialize' => '->super || \&serialize_fallback' ],          # object, { options } => value or ()
+   [ '&deserialize' => '->super || \&deserialize_meth' ],          # JSON => object
+   [ '&JSONschema' => '->super' ],                                 #
    [ '$construct_node' => 'undef' ],                               # Overload::Node
    [ '&construct' => '\&construct_object' ],                       # args ... => object
    [ '&parse' => '->super || \&parse_fallback' ],                  # "string" => object
@@ -64,14 +67,11 @@ use Polymake::Struct (
    [ '&init' => '->super || \&init_fallback' ],
    [ '$context_pkg' => 'undef' ],
    [ '$cppoptions' => 'undef' ],
-   [ '$help' => 'undef' ],              # InteractiveHelp (in interactive mode, when comments are supplied for user methods)
+   [ '$operators' => 'undef' ],
+   [ '$help' => 'undef' ],              # Help::Topic if loaded
 );
 
-declare @override_methods=qw( canonical equal isa coherent_type parse toString XMLdatatype init );
-
-declare @string_ops=map { $_ => eval <<"." } qw( . cmp eq ne lt le gt ge );
-sub { \$_[2] ? "\$_[1]" $_ "\$_[0]" : "\$_[0]" $_ "\$_[1]" }
-.
+declare @override_methods=qw( canonical equal isa coherent_type serialize JSONschema parse toString init );
 
 ####################################################################################
 #
@@ -88,6 +88,8 @@ sub new {
    define_function($pkg, ".type", $self_sub);
    if ($self->super) {
       $self->dimension=$self->super->dimension;
+      $self->params=$self->super->params;
+      $self->operators=$self->super->operators;
       if ($self->construct_node=$self->super->construct_node) {
          create_method_new($self);
       }
@@ -123,12 +125,12 @@ if ($] < 5.018) {
 }
 ##################################################################################
 sub add_constructor {
-   my ($self, $name, $code, $arg_types)=@_;
+   my ($self, $name, $label, $code, $arg_types)=@_;
    $self->construct_node //= do {
       create_method_new($self);
-      new Overload::Node(undef, undef, 0);
+      new_root Overload::Node;
    };
-   Overload::add_instance($self->pkg, ".$name", $code, undef, $arg_types, undef, $self->construct_node);
+   Overload::add_instance($self->pkg, ".$name", $label, $code, $arg_types, undef, $self->construct_node);
 }
 
 sub create_method_new : method {
@@ -183,6 +185,8 @@ sub concrete_typecheck : method {
    ? $self : undef
 }
 
+sub no_typecheck : method { croak( $_[0]->name, " inadvertently involved in overload resolution" ) }
+
 sub descend_to_generic {
    my ($self, $pkg)=@_;
    if (defined $pkg) {
@@ -196,9 +200,9 @@ sub descend_to_generic {
 ##################################################################################
 # used in overload resolution
 sub typecheck : method {
-   my ($self, $arg_list, $arg, $backtrack)=@_;
-   if (defined (my $obj_proto=Overload::fetch_type($arg))) {
-      $self->perform_typecheck->($arg_list, $obj_proto, $backtrack);
+   my ($self, $full_args, $args, $arg_index, $backtrack)=@_;
+   if (defined (my $obj_proto = Overload::fetch_type($args->[$arg_index]))) {
+      $self->perform_typecheck->($full_args, $obj_proto, $backtrack);
    }
 }
 ##################################################################################
@@ -220,32 +224,19 @@ sub add_upgrade_relations {
 #################################################################################
 # produce a name sufficient for reconstruction from a data file
 sub qualified_name {
-   my ($self, $in_app)=@_;
-   (not(defined($in_app) and $in_app==$self->application || $in_app->imported->{$self->application->name}) && $self->application->name . "::") . $self->name
-}
-#################################################################################
-sub locate_own_help_topic {
-   my ($self, $whence, $force)=@_;
-   return if !defined($self->application);
-   my $full_name=$self->full_name;
-   my $topic=$self->application->help->find($whence, $full_name) //
-             (defined($self->generic)
-              ? ($force ? croak( "internal error: asked to create a help node for a parametrized type instance" )
-                        : $self->application->help->find($whence, $self->generic->name))
-              : ($force ? $self->application->help->add([$whence, $full_name])
-                        : return));
-   if (!exists $topic->annex->{type}) {
-      weak($topic->annex->{type}=$self);
-      push @{$topic->related},
-           uniq( map { ($_, @{$_->related}) } grep { defined and $_ != $topic and !defined($_->spez_topic) } map { $_->help_topic }
-                 $whence eq "property_types" ? defined($self->super) ? $self->super : () : @{$self->super} );
-   }
-   $self->help=$topic;
+   my ($self, $in_app) = @_;
+   (not(defined($in_app) and $in_app == $self->application || $in_app->imported->{$self->application->name}) && $self->application->name . "::") .
+   $self->name
 }
 
+sub required_extensions {
+   my ($self, $exclude_bundled) = @_;
+   defined($self->extension) && !($exclude_bundled && $self->extension->is_bundled) ? ($self->extension) : ()
+}
+#################################################################################
 sub help_topic {
-   my $self=shift;
-   $self->help // locate_own_help_topic($self, "property_types", @_);
+   my $self = shift;
+   $self->help // ($self->application // return)->help->find_type_topic($self, "property_types", @_);
 }
 #################################################################################
 package Polymake::Core::PropertyParamedType;
@@ -258,25 +249,11 @@ use Polymake::Struct (
    [ '$application' => '#1->application' ],
    [ '$extension' => '#1->extension' ],
    [ '$super' => '#2 // #1' ],
-   [ '$generic' => '#1' ],
    [ '$params' => '#3' ],
+   [ '$generic' => '#1' ],
    '@derived_abstract',
 );
 
-use Polymake::Struct (
-   [ 'alt.constructor' => '_new_generic' ],
-   [ new => '$$$$$' ],
-   [ '$name' => '#1' ],
-   [ '$pkg' => '#2' ],
-   [ '$application' => '#3' ],
-   [ '$extension' => '$Application::extension' ],
-   [ '$super' => '#4' ],
-   [ '$generic' => 'undef' ],
-   [ '$params' => '#5' ],
-   [ '&abstract' => '\&type' ],
-   [ '&perform_typecheck' => 'undef' ],
-);
-
 ####################################################################################
 #
 #  Constructor
@@ -288,7 +265,7 @@ sub new {
    Struct::learn_package_retrieval($self, \&PropertyType::pkg);
    scan_params($self);
    unless ($self->abstract) {
-      if ($self->super && $self->super != $self->generic) {
+      if (defined($self->super) && $self->super != $self->generic) {
          # methods defined for the own generic class have precedence over those from the super class
          my $generic_index=Struct::get_field_index(\&generic);
          foreach my $method (@override_methods) {
@@ -301,9 +278,13 @@ sub new {
       define_function($self->pkg, "type", $self_sub, 1);
       define_function($self->pkg, ".type", $self_sub);
       $self->dimension=$self->super->dimension;
+      $self->operators=$self->super->operators;
       $self->construct_node=$self->super->construct_node;
       $self->create_method_new();
       establish_inheritance($self, $self->generic, $self->super);
+      if (defined($self->generic->cppoptions)) {
+         $self->application->cpp->add_template_instance($self, $self->generic, $nested_instantiation);
+      }
       $self->init->();
    }
    $self;
@@ -349,31 +330,41 @@ sub performs_deduction {
    $self->abstract && defined($self->perform_typecheck) && $self->perform_typecheck != \&context_sensitive_typecheck;
 }
 
-sub new_generic {
-   my ($self_pkg, $name, $pkg, $app, $params, $super, $subpkg)=@_;
-   my $self;
-   my $n_defaults=shift @$params;
-   my $min_params=@$params-$n_defaults;
-   my $param_index=-1;
-   my @param_holders=map { new ClassTypeParam($_, $pkg, $app, ++$param_index) } @$params;
-   my $symtab=get_symtab($pkg);
-   if (!defined($subpkg)) {
-      $self=_new_generic($self_pkg, $name, $pkg, $app, $super, \@param_holders);
-      if ($super) {
-         $self->dimension=$super->dimension;
-         if ($self->construct_node=$super->construct_node) {
-            $self->create_method_new();
-         }
-         push @{$super->derived_abstract}, $self;
-         weak($super->derived_abstract->[-1]);
-      }
-      $subpkg="props";
-   }
-   define_function($symtab, "_min_params", sub { $min_params });
+sub create_param_placeholders {
+   my ($pkg, $app, $params) = @_;
+   my $n_defaults = shift @$params;
+   my $min_params = @$params-$n_defaults;
+   define_function($pkg, "_min_params", sub { $min_params });
+   my $param_index = -1;
+   [ map { new ClassTypeParam($_, $pkg, $app, ++$param_index) } @$params ];
+}
 
-   no strict 'refs';
-   *{$app->pkg."::$subpkg\::$name\::"}=$symtab;
-   $self // \@param_holders;
+use Polymake::Struct (
+   [ 'alt.constructor' => '_new_generic' ],
+   [ new => '$$$$;$' ],
+   [ '$name' => '#1' ],
+   [ '$pkg' => '#2' ],
+   [ '$application' => '#3' ],
+   [ '$extension' => '$Extension::loading' ],
+   [ '$super' => '#5' ],
+   [ '$generic' => 'undef' ],
+   [ '$params' => 'create_param_placeholders(#2, #3, #4)' ],
+   [ '&abstract' => '\&type' ],
+   [ '&perform_typecheck' => 'undef' ],
+);
+
+sub new_generic {
+   my $self = &_new_generic;
+   if (my $super = $self->super) {
+      $self->dimension = $super->dimension;
+      $self->operators = $super->operators;
+      if ($self->construct_node = $super->construct_node) {
+         $self->create_method_new();
+      }
+      push @{$super->derived_abstract}, $self;
+      weak($super->derived_abstract->[-1]);
+   }
+   $self
 }
 
 #################################################################################
@@ -416,6 +407,17 @@ sub scan_params {
    }
 }
 #################################################################################
+sub find_super_type_param {
+   my ($self, $name) = @_;
+   my @result;
+   while (defined ($self = $self->super)) {
+      if (defined($self->params) and my ($param) = grep { $_->name eq $name } @{$self->params}) {
+         @result = ($self, $param);
+      }
+   }
+   @result;
+}
+#################################################################################
 sub full_name {
    my ($self)=@_;
    if ($self->abstract ? !$self->performs_deduction : !defined($self->params)) {
@@ -430,18 +432,17 @@ sub mangled_name {
    $1
 }
 
-# produce a name adhering to the XML name syntax rule, e.g. as a schema element identifier
-sub xml_name {
-   my ($self)=@_;
-   join(".", $self->name, map { $_->xml_name } @{$self->params})
-}
-
 # produce a name sufficient for reconstruction from a data file
 sub qualified_name {
-   my ($self, $in_app)=@_;
+   my ($self, $in_app) = @_;
    $in_app //= $self->application;
    &PropertyType::qualified_name . "<" . join(", ", map { $_->qualified_name($in_app) } @{$self->params}) . ">"
 }
+
+sub required_extensions {
+   my ($self, $include_bundled) = @_;
+   map { PropertyType::required_extensions($_, $include_bundled) } $self, @{$self->params}
+}
 #################################################################################
 # Update the list of independent extensions in $_[0].
 # Both arguments may be undef (equivalent to an empty list) or single Extension objects (equivalent to a one-element list)
@@ -500,7 +501,7 @@ use Polymake::Struct (
    [ '$name' => '#1' ],
    [ '$pkg' => '#2' ],
    [ '$application' => '#3' ],
-   [ '$extension' => '$Application::extension' ],
+   [ '$extension' => '$Extension::loading' ],
    [ '$super' => '#4' ],
    [ '$generic' => 'undef' ],
    [ '$param' => 'undef' ],
@@ -515,6 +516,9 @@ sub new {
    define_function($self->pkg, "type", $self_sub, 1);
    define_function($self->pkg, ".type", $self_sub);
    establish_inheritance($self, $self->generic, $self->super);
+   if (defined($self->generic->cppoptions)) {
+      $self->application->cpp->add_template_instance($self, $self->generic, $nested_instantiation);
+   }
    $self;
 }
 
@@ -542,6 +546,7 @@ use Polymake::Struct (
    [ '$super' => 'undef' ],
    [ '&abstract' => '\&extract_type' ],
    [ '&perform_typecheck' => '\&no_typecheck' ],
+   [ '$pkg' => '"UNIVERSAL"' ],
    [ '$context_pkg' => '#2' ],
    [ '$type_param_index' => '#4' ],
 );
@@ -553,9 +558,12 @@ sub extract_type : method {
    (is_object($_[1]) ? $_[1] : $_[1]->[0])->params->[$_[0]->type_param_index]
 }
 
-*concrete_type=\&PropertyParamedType::concrete_type;
+sub typecheck : method {
+   my ($self, $full_args, $args, $arg_index)=@_;
+   &extract_type->isa->($args->[$arg_index]) && $self;
+}
 
-sub no_typecheck : method { croak( $_[0]->name, " inadvertently involved in overload resolution" ) }
+*concrete_type=\&PropertyParamedType::concrete_type;
 
 sub full_name {
    my ($self)=@_;
@@ -651,7 +659,7 @@ sub typeof {
       instanceof FunctionTypeParam($_[1])
         or croak( "type_upgrade is only applicable to function type parameters" );
       state %inst_cache;
-      $inst_cache{ $_[1] } ||= &new;
+      $inst_cache{ $_[1] } //= &new;
 
    } elsif (@_==3) {
       # final typecheck clause
@@ -687,22 +695,25 @@ use Polymake::Struct (
    [ '$super' => 'undef' ],
    [ '$generic' => 'undef' ],
    [ '$params' => '[ #1 ]' ],
-   [ '$context_pkg' => '#1->context_pkg' ],
+   [ '$context_pkg' => 'undef' ],
    [ '&perform_typecheck' => '\&check_upgradable' ],
 );
 
 sub new {
    my $self=&_new;
-   $self->abstract= $self->params->[0]->abstract
-                    ? sub : method { $_[0]->params->[0]->abstract->($_[1]) }
-                    : sub : method { $_[0]->params->[0] };
+   if ($self->params->[0]->abstract) {
+      $self->abstract = sub : method { $_[0]->params->[0]->abstract->($_[1]) };
+      $self->context_pkg = $self->params->[0]->context_pkg;
+   } else {
+      $self->abstract = sub : method { $_[0]->params->[0] };
+   }
    $self;
 }
 
 sub typeof {
    @_==2 or croak( "type_upgrades_to requires exactly one type parameter" );
    state %inst_cache;
-   $inst_cache{ $_[1] } ||= &new;
+   $inst_cache{ $_[1] } //= &new;
 }
 
 sub check_upgradable : method {
@@ -717,6 +728,38 @@ sub check_upgradable : method {
 
 sub type_param_index : method { $_[0]->params->[0]->type_param_index }
 
+#################################################################################
+package Polymake::Core::PropertyType::ConvertTo;
+
+use Polymake::Struct (
+   [ '@ISA' => 'UpgradesTo' ],
+   [ new => '$' ],
+   [ '$name' => '"can_convert_to"' ],
+   [ '&perform_typecheck' => '\&no_typecheck' ],
+);
+
+sub typeof {
+   @_==2 or croak( "can_convert_to requires exactly one type parameter" );
+   state %inst_cache;
+   $inst_cache{ $_[1] } //= &new;
+}
+
+sub typecheck : method {
+   my ($self, $full_args, $args, $arg_index, $backtrack)=@_;
+   if (defined (my $target_proto=$self->abstract->($full_args))) {
+      my $given=$args->[$arg_index];
+      if ($target_proto->isa->($given)) {
+         return $self;
+      }
+      if (defined (my $converted=eval { $target_proto->construct->($given) })) {
+         push @$backtrack, sub { $args->[$arg_index]=$given };
+         $args->[$arg_index]=$converted;
+         return $self;
+      }
+   }
+   undef
+}
+
 #################################################################################
 package main;
 
@@ -725,6 +768,7 @@ sub UNIVERSAL::typeof_gen { @_==1 && $_[0] }
 
 *type_upgrades_to::=\%Polymake::Core::PropertyType::UpgradesTo::;
 *type_upgrade::=\%Polymake::Core::PropertyType::Upgrade::;
+*can_convert_to::=\%Polymake::Core::PropertyType::ConvertTo::;
 
 1
 
